--- old/usr/src/uts/sun4u/vm/mach_vm_dep.c	2016-06-30 22:03:59.399330037 -0400
+++ new/usr/src/uts/sun4u/vm/mach_vm_dep.c	2016-06-30 22:03:59.427558950 -0400
@@ -46,6 +46,7 @@
 #include <sys/memnode.h>
 #include <sys/mem_cage.h>
 #include <vm/vm_dep.h>
+#include <sys/random.h>
 
 #if defined(__sparcv9) && defined(SF_ERRATA_57)
 caddr_t errata57_limit;
@@ -137,6 +138,13 @@
 }
 
 /*
+ * The maximum amount a randomized mapping will be slewed.  We should perhaps
+ * arrange things so these tunables can be separate for mmap, mmapobj, and
+ * ld.so
+ */
+size_t aslr_max_map_skew = 256 * 1024 * 1024; /* 256MB */
+
+/*
  * map_addr_proc() is the routine called when the system is to
  * choose an address for the user.  We will pick an address
  * range which is just below the current stack limit.  The
@@ -265,6 +273,7 @@
 	 */
 	as_purge(as);
 	off = off & (align_amount - 1);
+
 	if (as_gap_aligned(as, len, &base, &slen, AH_HI, NULL, align_amount,
 	    PAGESIZE, off) == 0) {
 		caddr_t as_addr;
@@ -288,6 +297,28 @@
 			addr -= align_amount;
 		}
 
+		/*
+		 * If randomization is requested, slew the allocation
+		 * backwards, within the same gap, by a random amount.
+		 */
+		if (flags & _MAP_RANDOMIZE) {
+			uint32_t slew;
+			uint32_t maxslew;
+
+			(void) random_get_pseudo_bytes((uint8_t *)&slew,
+			    sizeof (slew));
+
+			maxslew = MIN(aslr_max_map_skew, (addr - base));
+			/*
+			 * Don't allow ASLR to cause mappings to fail below
+			 * because of SF erratum #57
+			 */
+			maxslew = MIN(maxslew, (addr - errata57_limit));
+
+			slew = slew % maxslew;
+			addr -= P2ALIGN(slew, align_amount);
+		}
+
 		ASSERT(addr > base);
 		ASSERT(addr + len < base + slen);
 		ASSERT(((uintptr_t)addr & (align_amount - 1l)) ==
@@ -349,9 +380,3 @@
 	/* not applicable to sun4u */
 	return (alloc_base);
 }
-
-size_t
-exec_get_spslew(void)
-{
-	return (0);
-}
